/*
 * Copyright (C) 2011 Ovea <dev@ovea.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.fn.dynaForm = function() {
        var set = this;
        return {
            toJSON: function() {
                var json = {};
                set.each(set.serializeArray(), function() {
                    var val = this.value;
                    if (json[this.name] !== undefined) {
                        if (!json[this.name].push) {
                            json[this.name] = [json[this.name]];
                        }
                        json[this.name].push(val);
                    } else if (this.value !== '') {
                        json[this.name] = val;
                    }
                });
                // force array for multiple selects
                set.filter('select[multiple]').each(function() {
                    if (json[this.name] !== undefined && !json[this.name].push) {
                        json[this.name] = [json[this.name]];
                    }
                });
                // force array for multiple checkboxes with same name
                var counts = {};
                set.filter(':checkbox').each(function() {
                    counts[this.name] = (counts[this.name] || 0) + 1
                });
                for (var name in counts) {
                    if (counts[name] > 1 && json[name] !== undefined && !json[name].push) {
                        json[name] = [json[name]];
                    }
                }
                return json;
            },
            fromJSON: function(values) {
                return set.each(function() {
                    if (values) {
                        for (var key in values) {
                            var inputs = $(':input[name=' + key + ']', this);
                            if (inputs.length) {
                                var input = inputs.get(0);
                                switch (input.nodeName.toLowerCase()) {
                                    case 'select':
                                    {
                                        var vals = values[key];
                                        if (!$.isArray(vals)) {
                                            vals = [vals];
                                        }
                                        inputs.find('option').filter(
                                            function() {
                                                return $.inArray(this.value, vals) != -1;
                                            }).attr('selected', true).trigger('click');
                                        inputs.trigger('change');
                                        break;
                                    }
                                    case 'input':
                                    {
                                        switch (input.type.toLowerCase()) {
                                            case 'text':
                                            {
                                                inputs.val(values[key]).trigger('change');
                                                break;
                                            }
                                            case 'radio':
                                            {
                                                inputs.filter('[value=' + values[key] + ']').attr('checked', true).trigger('click').trigger('change');
                                                break;
                                            }
                                            case 'checkbox':
                                            {
                                                var vals = values[key];
                                                if (!$.isArray(vals)) {
                                                    vals = [vals];
                                                }
                                                inputs.filter(
                                                    function() {
                                                        return $.inArray(this.value, vals) != -1;
                                                    }).attr('checked', true).trigger('click').trigger('change');
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                });
            }
        }
    }
})(jQuery);